@@ -2,19 +2,24 @@
 
 use strict;
 use warnings;
+use YAML;
 
-my (@info, @token_enum, @kind_enum);
+my (@info, @token_enum, @kind_enum, @syntax_enum, @type_to_info);
 foreach (<DATA>) {
     my ($kind, $type, $data) = split /\s+/;
-    push @info, { type => "$type", kind => "$kind", data => "$data" };
+    my $info = { type => "$type", kind => "$kind", data => "$data" };
+    push @info, $info;
     unless (grep { $_ eq $type } @token_enum) {
         push @token_enum, $type;
+        $type_to_info[scalar @token_enum - 1] = $info;
     }
     unless (grep { $_ eq $kind } @kind_enum) {
         push @kind_enum, $kind if ($kind);
     }
 }
 
+@syntax_enum = qw/Value Term Expr Stmt BlockStmt/;
+
 my $token_type = join ",\n", map { "\t$_" } @token_enum;
 my $token_kind = join ",\n", map { "\t$_" } @kind_enum;
 my $token_info = join ",\n", map {
@@ -22,15 +27,54 @@ my $token_info = join ",\n", map {
             $_->{type}, $_->{kind}, $_->{type}, $_->{data});
 } @info;
 
+my $type_to_info = join ",\n", map {
+    sprintf(qq|\t{Enum::Token::Type::%s, Enum::Token::Kind::%s, "%s", "%s"}|,
+            $_->{type}, $_->{kind}, $_->{type}, $_->{data});
+} @type_to_info;
+
+my %keyword_map;
+$keyword_map{$_->{data}}++ foreach @info;
+
+my $keywords = join "\n", map {
+    sprintf(qq|"%s", {Enum::Token::Type::%s, Enum::Token::Kind::%s, "%s", "%s"}|,
+            $_->{data}, $_->{type}, $_->{kind}, $_->{type}, $_->{data});
+} grep {
+    $keyword_map{$_->{data}} == 1
+} grep {
+    $_->{data}
+} @info;
+
 my $count = 0;
 my $token_type_enums = join ",\n", map {
     ' ' x 4 . "T_$_ => " . $count++;
 } @token_enum;
+
+$count = 0;
+my $syntax_type_enums = join ",\n", map {
+    ' ' x 4 . "T_$_ => " . $count++;
+} @syntax_enum;
+
 $count = 0;
 my $token_kind_enums = join ",\n", map {
     ' ' x 4 . "T_$_ => " . $count++;
 } @kind_enum;
 
+my %token_type_constants_map;
+my %token_kind_constants_map;
+my %syntax_type_constants_map;
+$count = 0;
+$token_type_constants_map{"T_$_"} = $count++ foreach @token_enum;
+$count = 0;
+$token_kind_constants_map{"T_$_"} = $count++ foreach @kind_enum;
+$count = 0;
+$syntax_type_constants_map{"T_$_"} = $count++ foreach @syntax_enum;
+
+my $constants = +{
+    token_type => \%token_type_constants_map,
+    syntax_type => \%syntax_type_constants_map,
+    token_kind => \%token_kind_constants_map
+};
+
 open(my $fh, '>', 'include/gen_token.hpp');
 print $fh <<"CODE";
 namespace Enum {
@@ -58,6 +102,10 @@ TokenInfo decl_tokens[] = {
 $token_info
 };
 
+TokenInfo type_to_info[] = {
+$type_to_info
+};
+
 CODE
 
 open($fh, '>', 'lib/Compiler/Lexer/Constants.pm');
@@ -74,11 +122,7 @@ package Compiler::Lexer::SyntaxType;
 use strict;
 use warnings;
 use constant {
-    T_Value     => 0,
-    T_Term      => 1,
-    T_Expr      => 2,
-    T_Stmt      => 3,
-    T_BlockStmt => 4
+$syntax_type_enums
 };
 1;
 
@@ -91,6 +135,27 @@ $token_kind_enums
 1;
 CODE
 
+open($fh, '>', 'gen/gen_constants.yaml');
+print $fh Dump $constants;
+
+open($fh, '>', 'gen/reserved_keywords.gperf');
+print $fh <<CODE;
+%{
+
+#include <lexer.hpp>
+
+typedef struct _ReservedKeyword {
+    const char *name;
+    TokenInfo info;
+} ReservedKeyword;
+%}
+
+ReservedKeyword;
+%%
+$keywords
+%%
+CODE
+
 close($fh);
 
 __DATA__
@@ -184,7 +249,6 @@ Function        	BuiltinFunc         	quotemeta
 Function        	BuiltinFunc         	split
 Function        	BuiltinFunc         	study
 Function        	BuiltinFunc         	pop
-Function        	BuiltinFunc         	pop
 Function        	BuiltinFunc         	push
 Function        	BuiltinFunc         	splice
 Function        	BuiltinFunc         	shift
@@ -226,7 +290,6 @@ Function        	BuiltinFunc         	telldir
 Function        	BuiltinFunc         	truncate
 Function        	BuiltinFunc         	warn
 Function        	BuiltinFunc         	write
-Function        	BuiltinFunc         	syswrite
 Function        	BuiltinFunc         	vec
 Function        	BuiltinFunc         	chdir
 Function        	BuiltinFunc         	chmod
@@ -304,7 +367,6 @@ Function        	BuiltinFunc         	endhostent
 Function        	BuiltinFunc         	endnetent
 Function        	BuiltinFunc         	endpwent
 Function        	BuiltinFunc         	getgrent
-Function        	BuiltinFunc         	getgrent
 Function        	BuiltinFunc         	getgrgid
 Function        	BuiltinFunc         	getgrnam
 Function        	BuiltinFunc         	getlogin
@@ -334,7 +396,6 @@ Function        	BuiltinFunc         	setservent
 Function        	BuiltinFunc         	gmtime
 Function        	BuiltinFunc         	localtime
 Function        	BuiltinFunc         	time
-Function        	BuiltinFunc         	times
 Function        	BuiltinFunc         	ref
 Function        	BuiltinFunc         	bless
 Function        	BuiltinFunc         	defined
