@@ -1,16 +1,54 @@
 #include <lexer.hpp>
 
 namespace TokenType = Enum::Token::Type;
-TokenManager::TokenManager(void) : max_token_size(0), idx(0)
+
+TokenManager::TokenManager(size_t script_size) : max_token_size(0), idx(0)
 {
+	size_t token_size = sizeof(Token);
+	size_t token_pool_size = script_size * token_size;
 	tokens = new Tokens();
-	size_t i = 0;
-	for (i = 0; decl_tokens[i].type != TokenType::Undefined; i++) {
-		type_to_info_map.insert(TypeMap::value_type(decl_tokens[i].type, decl_tokens[i]));
-		data_to_info_map.insert(TypeDataMap::value_type(decl_tokens[i].data, decl_tokens[i]));
+	pool = (TokenPool *)calloc(script_size, token_size);
+	head = pool;
+	undefined_info = getTokenInfo(TokenType::Undefined);
+}
+
+Token *TokenManager::at(size_t i)
+{
+	return head + i;
+}
+
+Token *TokenManager::nextToken(Token *tk)
+{
+	return (tk + 1 < pool) ? tk + 1 : NULL;
+}
+
+Token *TokenManager::previousToken(Token *tk)
+{
+	return (tk != head) ? tk - 1 : NULL;
+}
+
+Token *TokenManager::lastToken(void)
+{
+	return (head != pool) ? pool-1 : NULL;
+}
+
+Token *TokenManager::beforeLastToken(void)
+{
+	return (head + 2 <= pool) ? pool-2 : NULL;
+}
+
+size_t TokenManager::size(void)
+{
+	return (pool - head);
+}
+
+void TokenManager::dump(void)
+{
+	size_t size = pool - head;
+	for (size_t i = 0; i < size; i++) {
+		Token *tk = (head + i);
+		fprintf(stdout, "[%-12s] : %12s \n", tk->_data, tk->info.name);
 	}
-	type_to_info_map.insert(TypeMap::value_type(decl_tokens[i].type, decl_tokens[i]));
-	data_to_info_map.insert(TypeDataMap::value_type(decl_tokens[i].data, decl_tokens[i]));
 }
 
 Token *TokenManager::getTokenByBase(Token *base, int offset)
@@ -76,18 +114,6 @@ Token *TokenManager::afterNextToken(void)
 		tokens->at(wanted_idx) : NULL;
 }
 
-TokenInfo TokenManager::getTokenInfo(TokenType::Type type)
-{
-	TypeMap::iterator it = type_to_info_map.find(type);
-	return (it != type_to_info_map.end()) ? it->second : getTokenInfo(TokenType::Undefined);
-}
-
-TokenInfo TokenManager::getTokenInfo(const char *data)
-{
-	TypeDataMap::iterator it = data_to_info_map.find(data);
-	return (it != data_to_info_map.end()) ? it->second : getTokenInfo(TokenType::Undefined);
-}
-
 Token *TokenManager::next(void)
 {
 	this->idx++;
@@ -104,11 +130,6 @@ void TokenManager::remove(size_t idx)
 	this->tokens->erase(this->tokens->begin() + idx);
 }
 
-void TokenManager::add(Token *tk)
-{
-	this->tokens->add(tk);
-}
-
 Token *TokenManager::back(void)
 {
 	this->idx--;
@@ -134,71 +155,3 @@ bool ScriptManager::compare(int start, int len, std::string target)
 	}
 	return false;
 }
-
-char ScriptManager::getCharByOffset(int offset)
-{
-	size_t current_idx = this->idx;
-	int wanted_idx = current_idx + offset;
-	return (0 <= wanted_idx && (size_t)wanted_idx < script_size) ?
-		raw_script[wanted_idx] : EOL;
-}
-
-char ScriptManager::beforePreviousChar(void)
-{
-	size_t current_idx = this->idx;
-	int wanted_idx = current_idx - 2;
-	return (0 <= wanted_idx && (size_t)wanted_idx < script_size) ?
-		raw_script[wanted_idx] : EOL;
-}
-
-char ScriptManager::previousChar(void)
-{
-	size_t current_idx = this->idx;
-	int wanted_idx = current_idx - 1;
-	return (0 <= wanted_idx && (size_t)wanted_idx < script_size) ?
-		raw_script[wanted_idx] : EOL;
-}
-
-char ScriptManager::currentChar(void)
-{
-	return raw_script[idx];
-}
-
-char ScriptManager::nextChar(void)
-{
-	size_t current_idx = this->idx;
-	int wanted_idx = current_idx + 1;
-	return (0 <= wanted_idx && (size_t)wanted_idx < script_size) ?
-		raw_script[wanted_idx] : EOL;
-}
-
-char ScriptManager::afterNextChar(void)
-{
-	size_t current_idx = this->idx;
-	int wanted_idx = current_idx + 2;
-	return (0 <= wanted_idx && (size_t)wanted_idx < script_size) ?
-		raw_script[wanted_idx] : EOL;
-}
-
-char ScriptManager::next(void)
-{
-	this->idx++;
-	return currentChar();
-}
-
-char ScriptManager::forward(size_t progress)
-{
-	this->idx += progress;
-	return currentChar();
-}
-
-char ScriptManager::back(void)
-{
-	this->idx--;
-	return currentChar();
-}
-
-bool ScriptManager::end(void)
-{
-	return currentChar() == EOL;
-}
