@@ -16,6 +16,9 @@ Scanner::Scanner() :
 	const char *regex_replaces[] = {
 		"s", "y", "tr", NULL
 	};
+	const char *enable_regex_argument_funcs[] = {
+		"map", "grep", "split", NULL
+	};
 	const char *operators[] = {
 		"<=>", "**=", "//=", "||=", "&&=", "...", "$#{",
 		"$^A", "$^D", "$^E", "$^F", "$^G", "$^H", "$^I",
@@ -37,6 +40,9 @@ Scanner::Scanner() :
 	for (size_t i = 0; regex_prefixes[i] != NULL; i++) {
 		regex_prefix_map.insert(StringMap::value_type(regex_prefixes[i], ""));
 	}
+	for (size_t i = 0; regex_replaces[i] != NULL; i++) {
+		enable_regex_argument_func_map.insert(StringMap::value_type(enable_regex_argument_funcs[i], ""));
+	}
 	for (size_t i = 0; regex_replaces[i] != NULL; i++) {
 		regex_replace_map.insert(StringMap::value_type(regex_replaces[i], ""));
 	}
@@ -47,55 +53,66 @@ Scanner::Scanner() :
 
 Token *Scanner::scanQuote(LexContext *ctx, char quote)
 {
-	Token *ret = NULL;
-	char *token = ctx->buffer();
 	TokenManager *tmgr = ctx->tmgr;
-	if (isStringStarted) {
-		ret = new Token(string(token), ctx->finfo);
+	ScriptManager *smgr = ctx->smgr;
+	for (smgr->next(); !smgr->end(); smgr->next()) {
+		char ch = smgr->currentChar();
+		if (ch == '\n') {
+			ctx->writeBuffer(ch);
+			ctx->finfo.start_line_num++;
+			continue;
+		} else if (ch == quote) {
+			char prev_ch = smgr->previousChar();
+			char before_prev_ch = smgr->beforePreviousChar();
+			if ((prev_ch == '\\' && before_prev_ch == '\\') || prev_ch != '\\') break;
+			ctx->writeBuffer(ch);
+		} else {
+			ctx->writeBuffer(ch);
+		}
+	}
+	Token *prev_tk = ctx->tmgr->lastToken();
+	int idx = ctx->tmgr->size() - 2;
+	string prev_data = (prev_tk) ? string(prev_tk->_data) : "";
+	string before_prev_data = (idx >= 0) ? string(ctx->tmgr->beforeLastToken()->_data) : "";
+
+	char *token = ctx->buffer();
+	Token *ret = ctx->tmgr->new_Token(token, ctx->finfo);
+	switch (quote) {
+	case '\'':
+		ret->info = tmgr->getTokenInfo(TokenType::RawString);
+		break;
+	case '"':
+		ret->info = tmgr->getTokenInfo(TokenType::String);
+		break;
+	case '`':
+		ret->info = tmgr->getTokenInfo(TokenType::ExecString);
+		break;
+	default:
+		break;
+	}
+	ctx->clearBuffer();
+
+	if (prev_data == "<<" || (before_prev_data == "<<" && prev_data == "\\")) {
+		/* String is HereDocument */
+		here_document_tag = string(ret->_data);
+		here_document_tag_tk = ret;
+		if (here_document_tag == "") {
+			here_document_tag = "\n";
+			here_document_tag_tk->_data = "\n";
+		}
 		switch (quote) {
 		case '\'':
-			ret->info = tmgr->getTokenInfo(TokenType::RawString);
+			ret->info = tmgr->getTokenInfo(TokenType::HereDocumentRawTag);
 			break;
 		case '"':
-			ret->info = tmgr->getTokenInfo(TokenType::String);
+			ret->info = tmgr->getTokenInfo(TokenType::HereDocumentTag);
 			break;
 		case '`':
-			ret->info = tmgr->getTokenInfo(TokenType::ExecString);
+			ret->info = tmgr->getTokenInfo(TokenType::HereDocumentExecTag);
 			break;
 		default:
 			break;
 		}
-		ctx->clearBuffer();
-		Token *prev_tk = ctx->tmgr->tokens->lastToken();
-		int idx = ctx->tmgr->tokens->size() - 2;
-		string prev_data = (prev_tk) ? prev_tk->data : "";
-		string before_prev_data = (idx >= 0) ? ctx->tmgr->tokens->at(idx)->data : "";
-		if (prev_data == "<<" || (before_prev_data == "<<" && prev_data == "\\")) {
-			/* String is HereDocument */
-			here_document_tag = ret->data;
-			if (here_document_tag == "") {
-				here_document_tag = "\n";
-			}
-			switch (quote) {
-			case '\'':
-				ret->info = tmgr->getTokenInfo(TokenType::HereDocumentRawTag);
-				break;
-			case '"':
-				ret->info = tmgr->getTokenInfo(TokenType::HereDocumentTag);
-				break;
-			case '`':
-				ret->info = tmgr->getTokenInfo(TokenType::HereDocumentExecTag);
-				break;
-			default:
-				break;
-			}
-		}
-		isStringStarted = false;
-		skipFlag = false;
-	} else {
-		start_string_ch = quote;
-		isStringStarted = true;
-		skipFlag = true;
 	}
 	return ret;
 }
@@ -103,7 +120,7 @@ Token *Scanner::scanQuote(LexContext *ctx, char quote)
 bool Scanner::scanNegativeNumber(LexContext *ctx, char number)
 {
 	char num_buffer[2] = {0};
-	if (number != EOL && !isStringStarted) {
+	if (number != EOL) {
 		num_buffer[0] = number;
 		if (atoi(num_buffer) > 0 || number == '0') {
 			//negative number
@@ -117,28 +134,28 @@ bool Scanner::scanNegativeNumber(LexContext *ctx, char number)
 bool Scanner::isRegexStartDelim(LexContext *ctx, const StringMap &map)
 {
 	/* exclude { m } or { m => ... } or { m, ... } or *m or //m */
-	Token *before_prev_token = ctx->tmgr->tokens->lastToken();
-	string before_prev_data = (before_prev_token) ? before_prev_token->data : "";
+	string prev_data = string(ctx->buffer());
+	//... [before_prev_token] [prev_token] [symbol] ...
+	if (map.find(prev_data) == map.end()) return false;
+	Token *before_prev_token = ctx->tmgr->lastToken();
+	string before_prev_data = (before_prev_token) ? string(before_prev_token->_data) : "";
 	TokenType::Type before_prev_type = (before_prev_token) ?
 		before_prev_token->info.type : TokenType::Undefined;
 	TokenKind::Kind before_prev_kind = (before_prev_token) ?
 		before_prev_token->info.kind : TokenKind::Undefined;
-	string prev_data = string(ctx->buffer());
 	char symbol = ctx->smgr->currentChar();
-	//... [before_prev_token] [prev_token] [symbol] ...
 	if (before_prev_type == TokenType::RegDelim) return false; /* regex option */
 	if (before_prev_data == "*") return false;  /* glob */
 	if (before_prev_data == "&") return false;  /* function call */
 	if (before_prev_data == "::") return false; /* method call */
 	if (symbol == '}' || symbol == '=' || symbol == ')') return false;
 	if (before_prev_kind == TokenKind::Modifier) return false; /* dereference */
-	if (map.find(prev_data) != map.end()) return true;
-	return false;
+	return true;
 }
 
 bool Scanner::isRegexEndDelim(LexContext *ctx)
 {
-	Token *token = ctx->tmgr->tokens->lastToken();
+	Token *token = ctx->tmgr->lastToken();
 	TokenType::Type type = (token) ? token->info.type : TokenType::Undefined;
 	if (isRegexStarted) return true;
 	if (type == TokenType::RegExp) return true;
@@ -175,10 +192,10 @@ char Scanner::getRegexDelim(LexContext *ctx)
 
 bool Scanner::isPrototype(LexContext *ctx)
 {
-	Token *prev_token = ctx->tmgr->tokens->lastToken();
-	string prev_data = (prev_token) ? prev_token->data : "";
-	int idx = ctx->tmgr->tokens->size() - 2;
-	string before_prev_data = (idx >= 0) ? ctx->tmgr->tokens->at(idx)->data : "";
+	Token *prev_token = ctx->tmgr->lastToken();
+	string prev_data = (prev_token) ? string(prev_token->_data) : "";
+	int idx = ctx->tmgr->size() - 2;
+	string before_prev_data = (idx >= 0) ? string(ctx->tmgr->beforeLastToken()->_data) : "";
 	char symbol = ctx->smgr->currentChar();
 	if (symbol != '(') return false;
 	if (prev_data == "sub") return true;
@@ -188,9 +205,9 @@ bool Scanner::isPrototype(LexContext *ctx)
 
 bool Scanner::isHereDocument(LexContext *ctx, Token *tk)
 {
-	int idx = ctx->tmgr->tokens->size() - 2;
-	string prev_tk_data = (idx >= 0) ? ctx->tmgr->tokens->at(idx)->data : "";
-	string tk_data = (tk) ? tk->data : "";
+	int idx = ctx->tmgr->size() - 2;
+	string prev_tk_data = (idx >= 0) ? string(ctx->tmgr->beforeLastToken()->_data) : "";
+	string tk_data = (tk) ? string(tk->_data) : "";
 	char *token = ctx->buffer();
 	if ((tk_data == "<<" || (prev_tk_data == "<<" && tk_data == "\\")) &&
 		strtod(token, NULL) == 0 && string(token) != "0" &&
@@ -200,52 +217,52 @@ bool Scanner::isHereDocument(LexContext *ctx, Token *tk)
 	return false;
 }
 
-bool Scanner::isFormat(LexContext *ctx, Token *tk)
+bool Scanner::isFormat(LexContext *, Token *tk)
 {
-	return (tk->data == "format") ? true : false;
+	return (string(tk->_data) == "format") ? true : false;
 }
 
 bool Scanner::isRegexDelim(Token *prev_token, char symbol)
 {
-	const char *prev_data = (prev_token) ? cstr(prev_token->data) : "";
+	const char *prev_data = (prev_token) ? prev_token->_data : "";
 	/* [^0-9] && !"0" && !CONST && !{hash} && ![array] && !func() && !$var */
 	string prev_tk = string(prev_data);
 	if (regex_delim == 0 && prev_token && prev_token->info.type == TokenType::Undefined &&
 		(symbol != '-' && symbol != '=' && symbol != ',' && symbol != ')') &&
-		(prev_tk == "q"  || prev_tk == "qq" || prev_tk == "qw" ||
-		 prev_tk == "qx" || prev_tk == "qr" || prev_tk == "m")) {
+		regex_prefix_map.find(prev_tk) != regex_prefix_map.end()) {
 		return true;
 	}
-	if (symbol == '/' && (prev_tk == "xor" || prev_tk == "and")) return true;
 	if (symbol != '/') return false;
 	if (!prev_token) return true;
+	if (symbol == '/' && (prev_tk == "xor" || prev_tk == "and")) return true;
 	if (strtod(prev_data, NULL)) return false;
-	if (string(prev_data) == "0") return false;
+	if (prev_tk == "0") return false;
+	if (enable_regex_argument_func_map.find(prev_tk) != enable_regex_argument_func_map.end()) return true;
 	if (!isupper(prev_data[0]) && prev_data[0] != '_' &&
 		prev_data[0] != '}' && prev_data[0] != ']' && prev_data[0] != ')' &&
 		prev_data[0] != '$' && prev_data[0] != '@' && prev_data[0] != '%') {
-		if (isalpha(prev_data[0]) &&
-			(string(prev_data) != "split" && string(prev_data) != "grep" && string(prev_data) != "map") &&
-			string(prev_data) != "if" && string(prev_data) != "unless") return false;
+		if (isalpha(prev_data[0]) && prev_tk != "if" &&
+			prev_tk != "unless" && prev_tk != "ok") return false;
 		return true;
 	}
-	if (string(prev_data) == "split" || string(prev_data) == "grep" || string(prev_data) == "map") return true;
 	return false;
 }
 
-Token *Scanner::scanPrevSymbol(LexContext *ctx, char symbol)
+Token *Scanner::scanPrevSymbol(LexContext *ctx, char )
 {
 	char *token = ctx->buffer();
-	Token *ret = new Token(string(token), ctx->finfo);
 	TokenManager *tmgr = ctx->tmgr;
+	Token *ret = NULL;
 	if (isRegexStartDelim(ctx, regex_prefix_map)) {
 		//RegexPrefix
+		ret = ctx->tmgr->new_Token(token, ctx->finfo);
 		ret->info = tmgr->getTokenInfo(token);
 		regex_delim = getRegexDelim(ctx);
 		isRegexStarted = true;
 		skipFlag = true;
 	} else if (isRegexStartDelim(ctx, regex_replace_map)) {
 		//ReplaceRegexPrefix
+		ret = ctx->tmgr->new_Token(token, ctx->finfo);
 		ret->info = tmgr->getTokenInfo(token);
 		char delim = getRegexDelim(ctx);
 		regex_delim = delim;
@@ -253,14 +270,20 @@ Token *Scanner::scanPrevSymbol(LexContext *ctx, char symbol)
 		isRegexStarted = true;
 		skipFlag = true;
 	} else if (isPrototype(ctx)) {
+		ret = ctx->tmgr->new_Token(token, ctx->finfo);
+
 		isPrototypeStarted = true;
 		skipFlag = true;
 	} else {
-		Token *prev_before_tk = ctx->tmgr->tokens->lastToken();
+		Token *prev_before_tk = ctx->tmgr->lastToken();
 		if (isHereDocument(ctx, prev_before_tk)) {
 			/* Key is HereDocument */
-			here_document_tag = token;
+			ret = ctx->tmgr->new_Token(token, ctx->finfo);
+			here_document_tag = string(token);
+			here_document_tag_tk = ret;
 			ret->info = tmgr->getTokenInfo(TokenType::HereDocumentRawTag);
+		} else {
+			ret = ctx->tmgr->new_Token(token, ctx->finfo);
 		}
 	}
 	ctx->clearBuffer();
@@ -270,17 +293,11 @@ Token *Scanner::scanPrevSymbol(LexContext *ctx, char symbol)
 Token *Scanner::scanCurSymbol(LexContext *ctx, char symbol)
 {
 	Token *ret = NULL;
-	char *token = ctx->buffer();
-	char tmp[2] = {0};
-	tmp[0] = symbol;
 	TokenManager *tmgr = ctx->tmgr;
-	Token *prev_tk = tmgr->tokens->lastToken();
-	int idx = tmgr->tokens->size() - 2;
-	string prev_before = (idx >= 0) ? tmgr->tokens->at(idx)->data : "";
+	Token *prev_tk = ctx->tmgr->lastToken();
+	int idx = ctx->tmgr->size() - 2;
+	string prev_before = (idx >= 0) ? string(ctx->tmgr->beforeLastToken()->_data) : "";
 	if (prev_before != "sub" && isRegexDelim(prev_tk, symbol)) {
-		ret = new Token(string(tmp), ctx->finfo);
-		ret->info = tmgr->getTokenInfo(TokenType::RegDelim);
-		ctx->clearBuffer();
 		if (!isRegexEndDelim(ctx)) {
 			regex_delim = getRegexDelim(ctx);
 			isRegexStarted = true;
@@ -288,22 +305,30 @@ Token *Scanner::scanCurSymbol(LexContext *ctx, char symbol)
 		} else {
 			regex_delim = 0;
 		}
+		ctx->writeBuffer(symbol);
+		ret = ctx->tmgr->new_Token(ctx->buffer(), ctx->finfo);
+		ret->info = tmgr->getTokenInfo(TokenType::RegDelim);
+		ctx->clearBuffer();
 	} else if (isRegexEndDelim(ctx)) {
-		ret = new Token(string(tmp), ctx->finfo);
+		ctx->writeBuffer(symbol);
+		ret = ctx->tmgr->new_Token(ctx->buffer(), ctx->finfo);
 		ret->info = tmgr->getTokenInfo(TokenType::RegDelim);
 		ctx->clearBuffer();
 	} else if (symbol == '@' || symbol == '$' || symbol == '%') {// || symbol == '&') {
 		ctx->writeBuffer(symbol);
 	} else if (symbol == ';') {
-		ret = new Token(string(tmp), ctx->finfo);
+		ctx->writeBuffer(symbol);
+		ret = ctx->tmgr->new_Token(ctx->buffer(), ctx->finfo);
 		ctx->clearBuffer();
 	} else if (isPrototype(ctx)) {
-		ret = new Token(string(tmp), ctx->finfo);
+		ctx->writeBuffer(symbol);
+		ret = ctx->tmgr->new_Token(ctx->buffer(), ctx->finfo);
 		ctx->clearBuffer();
 		isPrototypeStarted = true;
 		skipFlag = true;
-	} else {
-		ret = new Token(string(tmp), ctx->finfo);
+	} else if (symbol != '\n') {
+		ctx->writeBuffer(symbol);
+		ret = ctx->tmgr->new_Token(ctx->buffer(), ctx->finfo);
 		ctx->clearBuffer();
 	}
 	return ret;
@@ -312,16 +337,17 @@ Token *Scanner::scanCurSymbol(LexContext *ctx, char symbol)
 Token *Scanner::scanTripleCharacterOperator(LexContext *ctx, char symbol, char next_ch, char after_next_ch)
 {
 	Token *ret = NULL;
-	char tmp[4] = {0};
-	tmp[0] = symbol;
-	tmp[1] = next_ch;
-	tmp[2] = after_next_ch;
-	if (operator_map.find(string(tmp)) != operator_map.end()) {
-		ret = new Token(string(tmp), ctx->finfo);
-		ret->info = ctx->tmgr->getTokenInfo(tmp);
+	char op[4] = { symbol, next_ch, after_next_ch, EOL };
+	if (triple_operator_map.in_word_set(op)) {// != operator_map.end()) {
+		ctx->writeBuffer(symbol);
+		ctx->writeBuffer(next_ch);
+		ctx->writeBuffer(after_next_ch);
+		ret = ctx->tmgr->new_Token(ctx->buffer(), ctx->finfo);
+		ret->info = ctx->tmgr->getTokenInfo(op);
+		ctx->clearBuffer();
 		ctx->progress = 2;
 	} else if (symbol == '$' && next_ch == '$') {
-		ret = new Token("$$", ctx->finfo);
+		ret = ctx->tmgr->new_Token((char *)"$$", ctx->finfo);
 		TokenManager *tmgr = ctx->tmgr;
 		ret->info = (isalpha(after_next_ch) || after_next_ch == '_') ? 
 			tmgr->getTokenInfo(TokenType::ShortScalarDereference) :
@@ -334,23 +360,25 @@ Token *Scanner::scanTripleCharacterOperator(LexContext *ctx, char symbol, char n
 Token *Scanner::scanDoubleCharacterOperator(LexContext *ctx, char symbol, char next_ch)
 {
 	Token *ret = NULL;
-	char tmp[3] = {0};
-	tmp[0] = symbol;
-	tmp[1] = next_ch;
-	if (operator_map.find(string(tmp)) != operator_map.end()) {
-		ret = new Token(string(tmp), ctx->finfo);
-		ret->info = ctx->tmgr->getTokenInfo(tmp);
+	char op[3] = { symbol, next_ch, EOL };
+	if (double_operator_map.in_word_set(op)) {
+		ctx->writeBuffer(symbol);
+		ctx->writeBuffer(next_ch);
+		ret = ctx->tmgr->new_Token(ctx->buffer(), ctx->finfo);
+		ret->info = ctx->tmgr->getTokenInfo(op);
+		ctx->clearBuffer();
 		ctx->progress = 1;
 	} else if (symbol == '/' && next_ch == '=') {
-		Token *prev_tk = ctx->tmgr->tokens->lastToken();
-		const char *prev_data = cstr(prev_tk->data);
+		Token *prev_tk = ctx->tmgr->lastToken();
+		const char *prev_data = prev_tk->_data;
 		/* '/=' is RegDelim + RegExp or DivEqual */
 		if (strtod(prev_data, NULL) != 0 || string(prev_data) == "0" || isupper(prev_data[0]) ||
 			prev_data[0] == '}' || prev_data[0] == ']' ||
 			prev_data[0] == ')' || prev_data[0] == '$') {
-			tmp[0] = symbol;
-			tmp[1] = next_ch;
-			ret = new Token(string(tmp), ctx->finfo);
+			ctx->writeBuffer(symbol);
+			ctx->writeBuffer(next_ch);
+			ret = ctx->tmgr->new_Token(ctx->buffer(), ctx->finfo);
+			ctx->clearBuffer();
 			ctx->progress = 1;
 		}
 	}
@@ -379,14 +407,18 @@ Token *Scanner::scanWordDelimiter(LexContext *ctx)
 	Token *ret = NULL;
 	if (ctx->existsBuffer()) {
 		char *token = ctx->buffer();
-		ret = new Token(string(token), ctx->finfo);
-		if (isHereDocument(ctx, tmgr->tokens->lastToken())) {
+		if (isHereDocument(ctx, ctx->tmgr->lastToken())) {
+			ret = ctx->tmgr->new_Token(token, ctx->finfo);
 			/* Key is HereDocument */
-			here_document_tag = token;
+			here_document_tag = string(token);
+			here_document_tag_tk = ret;
 			ret->info = tmgr->getTokenInfo(TokenType::HereDocumentRawTag);
-		} else if (isFormat(ctx, ret)) {
+		} else if (string(token) == "format") {
 			isFormatDeclared = true;
+			ret = ctx->tmgr->new_Token(token, ctx->finfo);
 			ret->info = tmgr->getTokenInfo(TokenType::FormatDecl);
+		} else if (token[0] != '\n' || token[1] != EOL) {
+			ret = ctx->tmgr->new_Token(token, ctx->finfo);
 		}
 		ctx->clearBuffer();
 	}
@@ -399,7 +431,7 @@ Token *Scanner::scanReference(LexContext *ctx)
 	char next_ch = ctx->smgr->nextChar();
 	if (next_ch == '$' || next_ch == '@' ||
 		next_ch == '%' || next_ch == '&') {
-		ret = new Token(string("\\"), ctx->finfo);
+		ret = ctx->tmgr->new_Token((char *)"\\", ctx->finfo);
 	}
 	return ret;
 }
@@ -409,12 +441,12 @@ Token *Scanner::scanSingleLineComment(LexContext *ctx)
 	Token *ret = NULL;
 	ScriptManager *smgr = ctx->smgr;
 	TokenManager *tmgr = ctx->tmgr;
-	if (ctx->existsBuffer()) tmgr->tokens->add(scanPrevSymbol(ctx, '#'));
-	Token *prev_tk = tmgr->tokens->lastToken();
+	if (ctx->existsBuffer()) tmgr->add(scanPrevSymbol(ctx, '#'));
+	Token *prev_tk = ctx->tmgr->lastToken();
 	TokenType::Type prev_type = (prev_tk) ?  prev_tk->info.type : TokenType::Undefined;
 	if (isRegexStarted || prev_type == TokenType::RegExp || prev_type ==  TokenType::RegReplaceTo) {
-		char tmp[2] = {'#'};
-		ret = new Token(string(tmp), ctx->finfo);
+		ctx->writeBuffer('#');
+		ret = ctx->tmgr->new_Token(ctx->buffer(), ctx->finfo);
 		ret->info = tmgr->getTokenInfo(TokenType::RegDelim);
 		ctx->clearBuffer();
 	} else {
@@ -427,8 +459,9 @@ Token *Scanner::scanSingleLineComment(LexContext *ctx)
 Token *Scanner::scanLineDelimiter(LexContext *ctx)
 {
 	Token *ret = scanWordDelimiter(ctx);
-	Token *last_tk = ctx->tmgr->tokens->lastToken();
-	string data = (ret) ? ret->data : (last_tk) ? last_tk->data : "";
+	Token *last_tk = ctx->tmgr->lastToken();
+	string data = (ret) ? string(ret->_data) :
+		(last_tk) ? string(last_tk->_data) : "";
 	if (isFormatDeclared && data == "=") {
 		isFormatDeclared = false;
 		isFormatStarted = true;
@@ -437,10 +470,16 @@ Token *Scanner::scanLineDelimiter(LexContext *ctx)
 		hereDocumentFlag = true;
 		skipFlag = true;
 	}
+	ctx->clearBuffer();
 	return ret;
 }
 
-#define NEXT() (*(src + i++))
+static inline char next(LexContext *ctx, char *src, size_t &i)
+{
+	ctx->writeBuffer((src+i)[0]);
+	return *(src + i++);
+}
+
 #define PREDICT() (*(src + i))
 #define is_number(ch) ('0' <= ch && ch <= '9')
 #define is_number_literal(ch) ((is_number(ch) || ch == '_') && ch != EOL)
@@ -461,14 +500,16 @@ Token *Scanner::scanVersionString(LexContext *ctx)
 {
 	TokenManager *tmgr = ctx->tmgr;
 	char *src = ctx->smgr->raw_script;
-	char *buf = ctx->buffer();
 	size_t i = ctx->smgr->idx;
 	char *begin = src + i;
-	char c = NEXT();
+	char c = next(ctx, src, i);//NEXT();
 	Token *token = NULL;
-	for (;(is_number(c) || c == '.') && c != EOL; c = NEXT()) {}
+	for (;(is_number(c) || c == '.') && c != EOL; c = next(ctx, src, i)) {}
 	i -= 1;
-	token = new Token(string(buf) + string(begin, src+i), ctx->finfo);
+	char *buf = ctx->buffer();
+	buf[ctx->buffer_idx-1] = EOL;
+
+	token = ctx->tmgr->new_Token(buf, ctx->finfo);
 	token->info = tmgr->getTokenInfo(TokenType::VersionString);
 	ctx->smgr->idx = --i;
 	return token;
@@ -480,16 +521,16 @@ Token *Scanner::scanNumber(LexContext *ctx)
 	char *src = ctx->smgr->raw_script;
 	size_t i = ctx->smgr->idx;
 	char *begin = src + i;
-	int c = NEXT();
+	int c = next(ctx, src, i);
 	Token *token = NULL;
 	assert((c == '.' || is_number(c)) && "It do not seem as Number");
 	bool isFloat = false;
 	if (is_number(c)) {
 		/* first char */
-		if (is_number_literal(c)) c = NEXT();
+		if (is_number_literal(c)) c = next(ctx, src, i);
 		/* second char is includes 'b' or 'x' */
-		if ((is_number(c) || c == 'b' || c == 'x' || c == '_') && c != EOL) c = NEXT();
-		for (;(is_number(c) || is_hexchar(c) || c == '_') && c != EOL; c = NEXT()) {}
+		if ((is_number(c) || c == 'b' || c == 'x' || c == '_') && c != EOL) c = next(ctx, src, i);
+		for (;(is_number(c) || is_hexchar(c) || c == '_') && c != EOL; c = next(ctx, src, i)) {}
 	}
 	if (c != '.' && c != 'e' && c != 'E') goto L_emit;
 	if (c == '.') {
@@ -498,17 +539,19 @@ Token *Scanner::scanNumber(LexContext *ctx)
 			goto L_emit; /* Number .. */
 		}
 		isFloat = true;
-		for (; is_number_literal(c); c = NEXT()) {}
+		for (; is_number_literal(c); c = next(ctx, src, i)) {}
 	}
 	if (c == 'e' || c == 'E') {
 		isFloat = true;
-		c = NEXT();
-		if (c == '+' || c == '-') c = NEXT();
-		for (; is_number_literal(c); c = NEXT()) {}
+		c = next(ctx, src, i);
+		if (c == '+' || c == '-') c = next(ctx, src, i);
+		for (; is_number_literal(c); c = next(ctx, src, i)) {}
 	}
 	L_emit:;
 	i -= 1;
-	token = new Token(string(begin, src+i), ctx->finfo);
+	char *buf = ctx->buffer();
+	buf[ctx->buffer_idx-1] = EOL;
+	token = ctx->tmgr->new_Token(buf, ctx->finfo);
 	token->info = isFloat ? tmgr->getTokenInfo(TokenType::Double) : tmgr->getTokenInfo(TokenType::Int);
 	ctx->smgr->idx = --i;
 	return token;
@@ -520,12 +563,14 @@ bool Scanner::isSkip(LexContext *ctx)
 {
 	using namespace TokenType;
 	bool ret = commentFlag;
+
 	ScriptManager *smgr = ctx->smgr;
 	TokenManager *tmgr = ctx->tmgr;
 	char *script = smgr->raw_script;
 	size_t idx = smgr->idx;
 	char prev_ch = smgr->previousChar();
 	char cur_ch = smgr->currentChar();
+
 	if (prev_ch == '\n' && cur_ch == '=' &&
 		isalnum(smgr->nextChar())) {
 		if (smgr->compare(1, 3, "cut")) {
@@ -539,28 +584,29 @@ bool Scanner::isSkip(LexContext *ctx)
 			commentFlag = true;
 			ret = true;
 		}
-	} else if (!hereDocumentFlag && prev_ch == '\n' && cur_ch == '_' &&
+	}
+	if (commentFlag) return ret;
+	if (prev_ch == '\n' && cur_ch == '_' && !hereDocumentFlag &&
 			   smgr->compare(0, 7, "__END__")) {
 		int progress_to_end = ctx->script_size - idx - 1;
 		ctx->progress = progress_to_end;
 		ret = false;
-	} else if (!hereDocumentFlag && prev_ch == '\n' && cur_ch == '_' &&
+	} else if (prev_ch == '\n' && cur_ch == '_' && !hereDocumentFlag &&
 			   smgr->compare(0, 8, "__DATA__")) {
 		int progress_to_end = ctx->script_size - idx - 1;
 		ctx->progress = progress_to_end;
 		ret = false;
 	}
-	if (commentFlag) return ret;
 	if (!skipFlag) return ret;
 
 	if (isFormatStarted) {
-		if (smgr->previousChar() == '\n' && smgr->currentChar() == '.') {
-			Token *tk = new Token(string(ctx->buffer()), ctx->finfo);
+		if (prev_ch == '\n' && cur_ch == '.') {
+			Token *tk = ctx->tmgr->new_Token(ctx->buffer(), ctx->finfo);
 			tk->info = tmgr->getTokenInfo(Format);
 			ctx->clearBuffer();
 			tmgr->add(tk);
 
-			tk = new Token(".", ctx->finfo);
+			tk = ctx->tmgr->new_Token((char *)".", ctx->finfo);
 			tk->info = tmgr->getTokenInfo(TokenType::FormatEnd);
 			tmgr->add(tk);
 
@@ -573,9 +619,9 @@ bool Scanner::isSkip(LexContext *ctx)
 			ret = true;
 		}
     } else if (isRegexStarted) {
-		if (smgr->previousChar() != '\\' ||
-			(smgr->previousChar() == '\\' && smgr->beforePreviousChar() == '\\')) {
-			switch (script[idx]) {
+		char before_prev_ch = smgr->beforePreviousChar();
+		if (prev_ch != '\\' || (prev_ch == '\\' && before_prev_ch == '\\')) {
+			switch (cur_ch) {
 			case '{': brace_count_inner_regex++;
 				break;
 			case '}': brace_count_inner_regex--;
@@ -592,17 +638,17 @@ bool Scanner::isSkip(LexContext *ctx)
 				break;
 			}
 		}
-		if (smgr->previousChar() == '\\' && smgr->beforePreviousChar() != '\\') {
-			ctx->writeBuffer(script[idx]);
+		if (prev_ch == '\\' && before_prev_ch != '\\') {
+			ctx->writeBuffer(cur_ch);
 			ret = true;
-		} else if (script[idx] != regex_delim && script[idx] != regex_middle_delim) {
-			ctx->writeBuffer(script[idx]);
+		} else if (cur_ch != regex_delim && cur_ch != regex_middle_delim) {
+			ctx->writeBuffer(cur_ch);
 			ret = true;
-		} else if (script[idx] == regex_middle_delim) {
+		} else if (cur_ch == regex_middle_delim) {
 			if ((regex_middle_delim == '}' && brace_count_inner_regex != 0) ||
 				(regex_middle_delim == ')' && cury_brace_count_inner_regex != 0) ||
 				(regex_middle_delim == ']' && bracket_count_inner_regex != 0)) {
-				ctx->writeBuffer(script[idx]);
+				ctx->writeBuffer(cur_ch);
 				ret = true;
 			} else {
 				Token *tk = NULL;
@@ -610,15 +656,17 @@ bool Scanner::isSkip(LexContext *ctx)
 					regex_middle_delim != '(' &&
 					regex_middle_delim != '<' &&
 					regex_middle_delim != '[') {
-					tk = new Token(string(ctx->buffer()), ctx->finfo);
+					tk = ctx->tmgr->new_Token(ctx->buffer(), ctx->finfo);
 					tk->info = tmgr->getTokenInfo(RegReplaceFrom);
 					ctx->clearBuffer();
 					tmgr->add(tk);
 				}
-				char tmp[2] = {regex_middle_delim, 0};
-				tk = new Token(string(tmp), ctx->finfo);
+				ctx->writeBuffer(regex_middle_delim);
+				tk = ctx->tmgr->new_Token(ctx->buffer(), ctx->finfo);
 				tk->info = tmgr->getTokenInfo(RegMiddleDelim);
+				ctx->clearBuffer();
 				tmgr->add(tk);
+
 				switch (regex_middle_delim) {
 				case '}':
 					regex_middle_delim = '{';
@@ -642,14 +690,15 @@ bool Scanner::isSkip(LexContext *ctx)
 			if ((regex_delim == '}' && brace_count_inner_regex != 0) ||
 				(regex_delim == ')' && cury_brace_count_inner_regex != 0) ||
 				(regex_delim == ']' && bracket_count_inner_regex != 0)) {
-				ctx->writeBuffer(script[idx]);
+				ctx->writeBuffer(cur_ch);
 				ret = true;
 			} else {
-				Token *tk = new Token(string(ctx->buffer()), ctx->finfo);
-				Token *prev_tk = tmgr->tokens->back();
+				Token *prev_tk = ctx->tmgr->lastToken();
+				Token *tk = ctx->tmgr->new_Token(ctx->buffer(), ctx->finfo);
 				tk->info = (prev_tk->info.type == RegMiddleDelim) ? tmgr->getTokenInfo(RegReplaceTo) : tmgr->getTokenInfo(RegExp);
 				ctx->clearBuffer();
 				tmgr->add(tk);
+
 				ret = false;
 				isRegexStarted = false;
 				skipFlag = false;
@@ -661,10 +710,11 @@ bool Scanner::isSkip(LexContext *ctx)
 		}
 	} else if (isPrototypeStarted) {
 		if (script[idx] == ')') {
-			Token *tk = new Token(string(ctx->buffer()), ctx->finfo);
+			Token *tk = ctx->tmgr->new_Token(ctx->buffer(), ctx->finfo);
 			tk->info = tmgr->getTokenInfo(Prototype);
 			ctx->clearBuffer();
 			tmgr->add(tk);
+
 			isPrototypeStarted = false;
 			skipFlag = false;
 			ret = false;
@@ -672,15 +722,6 @@ bool Scanner::isSkip(LexContext *ctx)
 			ctx->writeBuffer(script[idx]);
 			ret = true;
 		}
-	} else if (isStringStarted) {
-		if (script[idx] == start_string_ch &&
-			((smgr->previousChar() == '\\' && smgr->beforePreviousChar() == '\\') ||
-			 smgr->previousChar() != '\\')) {
-			ret = false;
-		} else {
-			ctx->writeBuffer(script[idx]);
-			ret = true;
-		}
 	} else if (hereDocumentFlag) {
 		size_t len = here_document_tag.size();
 		if (smgr->previousChar() == '\n' && idx + len < ctx->script_size) {
@@ -688,11 +729,12 @@ bool Scanner::isSkip(LexContext *ctx)
 			for (i = 0; i < len && script[idx + i] == here_document_tag.at(i); i++) {}
 			if (i == len) {
 				ctx->progress = len;
-				Token *tk = new Token(string(ctx->buffer()), ctx->finfo);
+				Token *tk = ctx->tmgr->new_Token(ctx->buffer(), ctx->finfo);
 				tk->info = tmgr->getTokenInfo(TokenType::HereDocument);
 				ctx->clearBuffer();
 				tmgr->add(tk);
-				tk = new Token(here_document_tag, ctx->finfo);
+
+				tk = ctx->tmgr->new_Token((char *)here_document_tag_tk->_data, ctx->finfo);
 				tk->info = tmgr->getTokenInfo(TokenType::HereDocumentEnd);
 				tmgr->add(tk);
 				ctx->finfo.start_line_num++;
