@@ -314,7 +314,8 @@ TokenInfo decl_tokens[] = {
 	{Enum::Token::Type::OurDecl, Enum::Token::Kind::Decl, "OurDecl", "our"},
 	{Enum::Token::Type::StateDecl, Enum::Token::Kind::Decl, "StateDecl", "state"},
 	{Enum::Token::Type::UseDecl, Enum::Token::Kind::Decl, "UseDecl", "use"},
-	{Enum::Token::Type::UsedName, Enum::Token::Kind::Term, "UsedName", ""},
+	{Enum::Token::Type::UsedName, Enum::Token::Kind::Module, "UsedName", ""},
+	{Enum::Token::Type::RequiredName, Enum::Token::Kind::Module, "RequiredName", ""},
 	{Enum::Token::Type::IfStmt, Enum::Token::Kind::Stmt, "IfStmt", "if"},
 	{Enum::Token::Type::ElseStmt, Enum::Token::Kind::Stmt, "ElseStmt", "else"},
 	{Enum::Token::Type::ElsifStmt, Enum::Token::Kind::Stmt, "ElsifStmt", "elsif"},
@@ -449,6 +450,210 @@ TokenInfo decl_tokens[] = {
 	{Enum::Token::Type::VersionString, Enum::Token::Kind::Term, "VersionString", ""},
 	{Enum::Token::Type::HereDocumentTag, Enum::Token::Kind::Term, "HereDocumentTag", ""},
 	{Enum::Token::Type::HereDocumentRawTag, Enum::Token::Kind::Term, "HereDocumentRawTag", ""},
+	{Enum::Token::Type::HereDocumentExecTag, Enum::Token::Kind::Term, "HereDocumentExecTag", ""},
+	{Enum::Token::Type::RawHereDocument, Enum::Token::Kind::Term, "RawHereDocument", ""},
+	{Enum::Token::Type::HereDocument, Enum::Token::Kind::Term, "HereDocument", ""},
+	{Enum::Token::Type::HereDocumentEnd, Enum::Token::Kind::Term, "HereDocumentEnd", ""},
+	{Enum::Token::Type::FormatDecl, Enum::Token::Kind::Decl, "FormatDecl", ""},
+	{Enum::Token::Type::Format, Enum::Token::Kind::Term, "Format", ""},
+	{Enum::Token::Type::FormatEnd, Enum::Token::Kind::Term, "FormatEnd", ""},
+	{Enum::Token::Type::Object, Enum::Token::Kind::Term, "Object", ""},
+	{Enum::Token::Type::RegExp, Enum::Token::Kind::Term, "RegExp", ""},
+	{Enum::Token::Type::Array, Enum::Token::Kind::Term, "Array", ""},
+	{Enum::Token::Type::Hash, Enum::Token::Kind::Term, "Hash", ""},
+	{Enum::Token::Type::Operator, Enum::Token::Kind::Operator, "Operator", ""},
+	{Enum::Token::Type::LocalVar, Enum::Token::Kind::Term, "LocalVar", ""},
+	{Enum::Token::Type::LocalArrayVar, Enum::Token::Kind::Term, "LocalArrayVar", ""},
+	{Enum::Token::Type::LocalHashVar, Enum::Token::Kind::Term, "LocalHashVar", ""},
+	{Enum::Token::Type::GlobalVar, Enum::Token::Kind::Term, "GlobalVar", ""},
+	{Enum::Token::Type::GlobalArrayVar, Enum::Token::Kind::Term, "GlobalArrayVar", ""},
+	{Enum::Token::Type::GlobalHashVar, Enum::Token::Kind::Term, "GlobalHashVar", ""},
+	{Enum::Token::Type::ArrayRef, Enum::Token::Kind::Ref, "ArrayRef", ""},
+	{Enum::Token::Type::HashRef, Enum::Token::Kind::Ref, "HashRef", ""},
+	{Enum::Token::Type::ArrayAt, Enum::Token::Kind::Get, "ArrayAt", ""},
+	{Enum::Token::Type::HashAt, Enum::Token::Kind::Get, "HashAt", ""},
+	{Enum::Token::Type::ArraySet, Enum::Token::Kind::Set, "ArraySet", ""},
+	{Enum::Token::Type::HashSet, Enum::Token::Kind::Set, "HashSet", ""},
+	{Enum::Token::Type::Function, Enum::Token::Kind::Decl, "Function", ""},
+	{Enum::Token::Type::Call, Enum::Token::Kind::Function, "Call", ""},
+	{Enum::Token::Type::Argument, Enum::Token::Kind::Term, "Argument", ""},
+	{Enum::Token::Type::List, Enum::Token::Kind::Term, "List", ""},
+	{Enum::Token::Type::Default, Enum::Token::Kind::Term, "Default", "undef"},
+	{Enum::Token::Type::Undefined, Enum::Token::Kind::Undefined, "Undefined", ""}
+};
+
+TokenInfo type_to_info[] = {
+	{Enum::Token::Type::Return, Enum::Token::Kind::Return, "Return", "return"},
+	{Enum::Token::Type::Add, Enum::Token::Kind::Operator, "Add", "+"},
+	{Enum::Token::Type::Sub, Enum::Token::Kind::Operator, "Sub", "-"},
+	{Enum::Token::Type::Mul, Enum::Token::Kind::Operator, "Mul", "*"},
+	{Enum::Token::Type::Div, Enum::Token::Kind::Operator, "Div", "/"},
+	{Enum::Token::Type::Mod, Enum::Token::Kind::Operator, "Mod", "%"},
+	{Enum::Token::Type::ThreeTermOperator, Enum::Token::Kind::Operator, "ThreeTermOperator", "?"},
+	{Enum::Token::Type::Greater, Enum::Token::Kind::Operator, "Greater", ">"},
+	{Enum::Token::Type::Less, Enum::Token::Kind::Operator, "Less", "<"},
+	{Enum::Token::Type::StringAdd, Enum::Token::Kind::Operator, "StringAdd", "."},
+	{Enum::Token::Type::Ref, Enum::Token::Kind::Operator, "Ref", "\\"},
+	{Enum::Token::Type::Glob, Enum::Token::Kind::Operator, "Glob", "*"},
+	{Enum::Token::Type::BitNot, Enum::Token::Kind::Operator, "BitNot", "~"},
+	{Enum::Token::Type::BitOr, Enum::Token::Kind::Operator, "BitOr", "|"},
+	{Enum::Token::Type::Or, Enum::Token::Kind::Operator, "Or", "or"},
+	{Enum::Token::Type::BitAnd, Enum::Token::Kind::Operator, "BitAnd", "&"},
+	{Enum::Token::Type::And, Enum::Token::Kind::Operator, "And", "and"},
+	{Enum::Token::Type::BitXOr, Enum::Token::Kind::Operator, "BitXOr", "^"},
+	{Enum::Token::Type::XOr, Enum::Token::Kind::Operator, "XOr", "xor"},
+	{Enum::Token::Type::StringMul, Enum::Token::Kind::Operator, "StringMul", "x"},
+	{Enum::Token::Type::AddEqual, Enum::Token::Kind::Assign, "AddEqual", "+="},
+	{Enum::Token::Type::SubEqual, Enum::Token::Kind::Assign, "SubEqual", "-="},
+	{Enum::Token::Type::MulEqual, Enum::Token::Kind::Assign, "MulEqual", "*="},
+	{Enum::Token::Type::DivEqual, Enum::Token::Kind::Assign, "DivEqual", "/="},
+	{Enum::Token::Type::ModEqual, Enum::Token::Kind::Assign, "ModEqual", "%="},
+	{Enum::Token::Type::GreaterEqual, Enum::Token::Kind::Operator, "GreaterEqual", ">="},
+	{Enum::Token::Type::LessEqual, Enum::Token::Kind::Operator, "LessEqual", "<="},
+	{Enum::Token::Type::EqualEqual, Enum::Token::Kind::Operator, "EqualEqual", "=="},
+	{Enum::Token::Type::Diamond, Enum::Token::Kind::Operator, "Diamond", "<>"},
+	{Enum::Token::Type::Compare, Enum::Token::Kind::Operator, "Compare", "<=>"},
+	{Enum::Token::Type::PolymorphicCompare, Enum::Token::Kind::Operator, "PolymorphicCompare", "~~"},
+	{Enum::Token::Type::RegOK, Enum::Token::Kind::Operator, "RegOK", "=~"},
+	{Enum::Token::Type::RegNot, Enum::Token::Kind::Operator, "RegNot", "!~"},
+	{Enum::Token::Type::NotEqual, Enum::Token::Kind::Operator, "NotEqual", "!="},
+	{Enum::Token::Type::ArraySize, Enum::Token::Kind::Operator, "ArraySize", "$#"},
+	{Enum::Token::Type::StringLess, Enum::Token::Kind::Operator, "StringLess", "lt"},
+	{Enum::Token::Type::StringLessEqual, Enum::Token::Kind::Operator, "StringLessEqual", "le"},
+	{Enum::Token::Type::StringGreater, Enum::Token::Kind::Operator, "StringGreater", "gt"},
+	{Enum::Token::Type::StringGreaterEqual, Enum::Token::Kind::Operator, "StringGreaterEqual", "ge"},
+	{Enum::Token::Type::StringEqual, Enum::Token::Kind::Operator, "StringEqual", "eq"},
+	{Enum::Token::Type::StringNotEqual, Enum::Token::Kind::Operator, "StringNotEqual", "ne"},
+	{Enum::Token::Type::StringCompare, Enum::Token::Kind::Operator, "StringCompare", "cmp"},
+	{Enum::Token::Type::Inc, Enum::Token::Kind::Operator, "Inc", "++"},
+	{Enum::Token::Type::Dec, Enum::Token::Kind::Operator, "Dec", "--"},
+	{Enum::Token::Type::Exp, Enum::Token::Kind::Operator, "Exp", "**"},
+	{Enum::Token::Type::PowerEqual, Enum::Token::Kind::Assign, "PowerEqual", "**="},
+	{Enum::Token::Type::DefaultEqual, Enum::Token::Kind::Assign, "DefaultEqual", "//="},
+	{Enum::Token::Type::LeftShift, Enum::Token::Kind::Operator, "LeftShift", "<<"},
+	{Enum::Token::Type::RightShift, Enum::Token::Kind::Operator, "RightShift", ">>"},
+	{Enum::Token::Type::AndBitEqual, Enum::Token::Kind::Assign, "AndBitEqual", "&="},
+	{Enum::Token::Type::OrBitEqual, Enum::Token::Kind::Assign, "OrBitEqual", "|="},
+	{Enum::Token::Type::NotBitEqual, Enum::Token::Kind::Assign, "NotBitEqual", "^="},
+	{Enum::Token::Type::OrEqual, Enum::Token::Kind::Assign, "OrEqual", "||="},
+	{Enum::Token::Type::AndEqual, Enum::Token::Kind::Assign, "AndEqual", "&&="},
+	{Enum::Token::Type::Slice, Enum::Token::Kind::Operator, "Slice", ".."},
+	{Enum::Token::Type::DefaultOperator, Enum::Token::Kind::Operator, "DefaultOperator", "//"},
+	{Enum::Token::Type::ToDo, Enum::Token::Kind::Operator, "ToDo", "..."},
+	{Enum::Token::Type::VarDecl, Enum::Token::Kind::Decl, "VarDecl", "my"},
+	{Enum::Token::Type::FunctionDecl, Enum::Token::Kind::Decl, "FunctionDecl", "sub"},
+	{Enum::Token::Type::Method, Enum::Token::Kind::Function, "Method", ""},
+	{Enum::Token::Type::Assign, Enum::Token::Kind::Assign, "Assign", "="},
+	{Enum::Token::Type::Is, Enum::Token::Kind::SingleTerm, "Is", ""},
+	{Enum::Token::Type::IsNot, Enum::Token::Kind::SingleTerm, "IsNot", "!"},
+	{Enum::Token::Type::Not, Enum::Token::Kind::SingleTerm, "Not", "not"},
+	{Enum::Token::Type::BuiltinFunc, Enum::Token::Kind::Function, "BuiltinFunc", "chomp"},
+	{Enum::Token::Type::RequireDecl, Enum::Token::Kind::Decl, "RequireDecl", "require"},
+	{Enum::Token::Type::Import, Enum::Token::Kind::Import, "Import", "import"},
+	{Enum::Token::Type::SpecificKeyword, Enum::Token::Kind::SpecificKeyword, "SpecificKeyword", "__PACKAGE__"},
+	{Enum::Token::Type::DataWord, Enum::Token::Kind::DataWord, "DataWord", "__DATA__"},
+	{Enum::Token::Type::ModWord, Enum::Token::Kind::ModWord, "ModWord", "BEGIN"},
+	{Enum::Token::Type::AUTOLOAD, Enum::Token::Kind::AUTOLOAD, "AUTOLOAD", "AUTOLOAD"},
+	{Enum::Token::Type::CORE, Enum::Token::Kind::CORE, "CORE", "CORE"},
+	{Enum::Token::Type::DESTROY, Enum::Token::Kind::DESTROY, "DESTROY", "DESTROY"},
+	{Enum::Token::Type::STDIN, Enum::Token::Kind::Handle, "STDIN", "STDIN"},
+	{Enum::Token::Type::STDOUT, Enum::Token::Kind::Handle, "STDOUT", "STDOUT"},
+	{Enum::Token::Type::STDERR, Enum::Token::Kind::Handle, "STDERR", "STDERR"},
+	{Enum::Token::Type::Redo, Enum::Token::Kind::Redo, "Redo", "redo"},
+	{Enum::Token::Type::Next, Enum::Token::Kind::Next, "Next", "next"},
+	{Enum::Token::Type::Last, Enum::Token::Kind::Last, "Last", "last"},
+	{Enum::Token::Type::Goto, Enum::Token::Kind::Goto, "Goto", "goto"},
+	{Enum::Token::Type::Continue, Enum::Token::Kind::Continue, "Continue", "continue"},
+	{Enum::Token::Type::Do, Enum::Token::Kind::Do, "Do", "do"},
+	{Enum::Token::Type::Break, Enum::Token::Kind::Break, "Break", "break"},
+	{Enum::Token::Type::Handle, Enum::Token::Kind::Handle, "Handle", "-b"},
+	{Enum::Token::Type::LocalDecl, Enum::Token::Kind::Decl, "LocalDecl", "local"},
+	{Enum::Token::Type::OurDecl, Enum::Token::Kind::Decl, "OurDecl", "our"},
+	{Enum::Token::Type::StateDecl, Enum::Token::Kind::Decl, "StateDecl", "state"},
+	{Enum::Token::Type::UseDecl, Enum::Token::Kind::Decl, "UseDecl", "use"},
+	{Enum::Token::Type::UsedName, Enum::Token::Kind::Module, "UsedName", ""},
+	{Enum::Token::Type::RequiredName, Enum::Token::Kind::Module, "RequiredName", ""},
+	{Enum::Token::Type::IfStmt, Enum::Token::Kind::Stmt, "IfStmt", "if"},
+	{Enum::Token::Type::ElseStmt, Enum::Token::Kind::Stmt, "ElseStmt", "else"},
+	{Enum::Token::Type::ElsifStmt, Enum::Token::Kind::Stmt, "ElsifStmt", "elsif"},
+	{Enum::Token::Type::UnlessStmt, Enum::Token::Kind::Stmt, "UnlessStmt", "unless"},
+	{Enum::Token::Type::UntilStmt, Enum::Token::Kind::Stmt, "UntilStmt", "until"},
+	{Enum::Token::Type::WhenStmt, Enum::Token::Kind::Stmt, "WhenStmt", "when"},
+	{Enum::Token::Type::GivenStmt, Enum::Token::Kind::Stmt, "GivenStmt", "given"},
+	{Enum::Token::Type::DefaultStmt, Enum::Token::Kind::DefaultStmt, "DefaultStmt", "default"},
+	{Enum::Token::Type::Comma, Enum::Token::Kind::Comma, "Comma", ","},
+	{Enum::Token::Type::Colon, Enum::Token::Kind::Colon, "Colon", ":"},
+	{Enum::Token::Type::SemiColon, Enum::Token::Kind::StmtEnd, "SemiColon", ";"},
+	{Enum::Token::Type::LeftParenthesis, Enum::Token::Kind::Symbol, "LeftParenthesis", "("},
+	{Enum::Token::Type::RightParenthesis, Enum::Token::Kind::Symbol, "RightParenthesis", ")"},
+	{Enum::Token::Type::LeftBrace, Enum::Token::Kind::Symbol, "LeftBrace", "{"},
+	{Enum::Token::Type::RightBrace, Enum::Token::Kind::Symbol, "RightBrace", "}"},
+	{Enum::Token::Type::LeftBracket, Enum::Token::Kind::Symbol, "LeftBracket", "["},
+	{Enum::Token::Type::RightBracket, Enum::Token::Kind::Symbol, "RightBracket", "]"},
+	{Enum::Token::Type::ArrayDereference, Enum::Token::Kind::Modifier, "ArrayDereference", "@{"},
+	{Enum::Token::Type::HashDereference, Enum::Token::Kind::Modifier, "HashDereference", "%{"},
+	{Enum::Token::Type::ScalarDereference, Enum::Token::Kind::Modifier, "ScalarDereference", "${"},
+	{Enum::Token::Type::ShortScalarDereference, Enum::Token::Kind::Modifier, "ShortScalarDereference", ""},
+	{Enum::Token::Type::ShortArrayDereference, Enum::Token::Kind::Modifier, "ShortArrayDereference", "@$"},
+	{Enum::Token::Type::ShortHashDereference, Enum::Token::Kind::Modifier, "ShortHashDereference", "%$"},
+	{Enum::Token::Type::ShortCodeDereference, Enum::Token::Kind::Modifier, "ShortCodeDereference", "&$"},
+	{Enum::Token::Type::ArraySizeDereference, Enum::Token::Kind::Modifier, "ArraySizeDereference", "$#{"},
+	{Enum::Token::Type::Key, Enum::Token::Kind::Term, "Key", ""},
+	{Enum::Token::Type::BareWord, Enum::Token::Kind::Term, "BareWord", ""},
+	{Enum::Token::Type::Arrow, Enum::Token::Kind::Operator, "Arrow", "=>"},
+	{Enum::Token::Type::Pointer, Enum::Token::Kind::Operator, "Pointer", "->"},
+	{Enum::Token::Type::NamespaceResolver, Enum::Token::Kind::Operator, "NamespaceResolver", "::"},
+	{Enum::Token::Type::Namespace, Enum::Token::Kind::Namespace, "Namespace", ""},
+	{Enum::Token::Type::Package, Enum::Token::Kind::Package, "Package", "package"},
+	{Enum::Token::Type::Class, Enum::Token::Kind::Class, "Class", ""},
+	{Enum::Token::Type::CallDecl, Enum::Token::Kind::Decl, "CallDecl", "&"},
+	{Enum::Token::Type::CodeRef, Enum::Token::Kind::SingleTerm, "CodeRef", "\\&"},
+	{Enum::Token::Type::WhileStmt, Enum::Token::Kind::Stmt, "WhileStmt", "while"},
+	{Enum::Token::Type::ForStmt, Enum::Token::Kind::Stmt, "ForStmt", "for"},
+	{Enum::Token::Type::ForeachStmt, Enum::Token::Kind::Stmt, "ForeachStmt", "foreach"},
+	{Enum::Token::Type::Annotation, Enum::Token::Kind::Annotation, "Annotation", "#@"},
+	{Enum::Token::Type::ArgumentArray, Enum::Token::Kind::Term, "ArgumentArray", "@_"},
+	{Enum::Token::Type::SpecificValue, Enum::Token::Kind::Term, "SpecificValue", "$_"},
+	{Enum::Token::Type::ConstValue, Enum::Token::Kind::Term, "ConstValue", ""},
+	{Enum::Token::Type::ProgramArgument, Enum::Token::Kind::Term, "ProgramArgument", "@ARGV"},
+	{Enum::Token::Type::LibraryDirectories, Enum::Token::Kind::Term, "LibraryDirectories", "@INC"},
+	{Enum::Token::Type::Environment, Enum::Token::Kind::Term, "Environment", "%ENV"},
+	{Enum::Token::Type::Include, Enum::Token::Kind::Term, "Include", "%INC"},
+	{Enum::Token::Type::Signal, Enum::Token::Kind::Term, "Signal", "%SIG"},
+	{Enum::Token::Type::RegOpt, Enum::Token::Kind::RegOpt, "RegOpt", ""},
+	{Enum::Token::Type::RegQuote, Enum::Token::Kind::RegPrefix, "RegQuote", "q"},
+	{Enum::Token::Type::RegDoubleQuote, Enum::Token::Kind::RegPrefix, "RegDoubleQuote", "qq"},
+	{Enum::Token::Type::RegList, Enum::Token::Kind::RegPrefix, "RegList", "qw"},
+	{Enum::Token::Type::RegExec, Enum::Token::Kind::RegPrefix, "RegExec", "qx"},
+	{Enum::Token::Type::RegDecl, Enum::Token::Kind::RegPrefix, "RegDecl", "qr"},
+	{Enum::Token::Type::RegMatch, Enum::Token::Kind::RegPrefix, "RegMatch", "m"},
+	{Enum::Token::Type::RegDelim, Enum::Token::Kind::Term, "RegDelim", ""},
+	{Enum::Token::Type::RegMiddleDelim, Enum::Token::Kind::Term, "RegMiddleDelim", ""},
+	{Enum::Token::Type::RegAllReplace, Enum::Token::Kind::RegReplacePrefix, "RegAllReplace", "tr"},
+	{Enum::Token::Type::RegReplace, Enum::Token::Kind::RegReplacePrefix, "RegReplace", "s"},
+	{Enum::Token::Type::RegReplaceFrom, Enum::Token::Kind::Term, "RegReplaceFrom", ""},
+	{Enum::Token::Type::RegReplaceTo, Enum::Token::Kind::Term, "RegReplaceTo", ""},
+	{Enum::Token::Type::FieldDecl, Enum::Token::Kind::Decl, "FieldDecl", ""},
+	{Enum::Token::Type::TypeRef, Enum::Token::Kind::Ref, "TypeRef", ""},
+	{Enum::Token::Type::LabelRef, Enum::Token::Kind::Ref, "LabelRef", ""},
+	{Enum::Token::Type::LocalVarDecl, Enum::Token::Kind::Decl, "LocalVarDecl", ""},
+	{Enum::Token::Type::GlobalVarDecl, Enum::Token::Kind::Decl, "GlobalVarDecl", ""},
+	{Enum::Token::Type::MultiLocalVarDecl, Enum::Token::Kind::Decl, "MultiLocalVarDecl", ""},
+	{Enum::Token::Type::MultiGlobalVarDecl, Enum::Token::Kind::Decl, "MultiGlobalVarDecl", ""},
+	{Enum::Token::Type::Prototype, Enum::Token::Kind::Term, "Prototype", ""},
+	{Enum::Token::Type::Var, Enum::Token::Kind::Term, "Var", ""},
+	{Enum::Token::Type::CodeVar, Enum::Token::Kind::Term, "CodeVar", ""},
+	{Enum::Token::Type::ArrayVar, Enum::Token::Kind::Term, "ArrayVar", ""},
+	{Enum::Token::Type::HashVar, Enum::Token::Kind::Term, "HashVar", ""},
+	{Enum::Token::Type::Int, Enum::Token::Kind::Term, "Int", ""},
+	{Enum::Token::Type::Double, Enum::Token::Kind::Term, "Double", ""},
+	{Enum::Token::Type::String, Enum::Token::Kind::Term, "String", ""},
+	{Enum::Token::Type::RawString, Enum::Token::Kind::Term, "RawString", ""},
+	{Enum::Token::Type::ExecString, Enum::Token::Kind::Term, "ExecString", ""},
+	{Enum::Token::Type::VersionString, Enum::Token::Kind::Term, "VersionString", ""},
+	{Enum::Token::Type::HereDocumentTag, Enum::Token::Kind::Term, "HereDocumentTag", ""},
+	{Enum::Token::Type::HereDocumentRawTag, Enum::Token::Kind::Term, "HereDocumentRawTag", ""},
+	{Enum::Token::Type::HereDocumentExecTag, Enum::Token::Kind::Term, "HereDocumentExecTag", ""},
 	{Enum::Token::Type::RawHereDocument, Enum::Token::Kind::Term, "RawHereDocument", ""},
 	{Enum::Token::Type::HereDocument, Enum::Token::Kind::Term, "HereDocument", ""},
 	{Enum::Token::Type::HereDocumentEnd, Enum::Token::Kind::Term, "HereDocumentEnd", ""},
